// C++ headers
#include <iostream>
#include <fstream>

// G4 header defining G4 types
#include "globals.hh"

// G4 headers
#include "G4ParticleTable.hh"
#include "G4ParticleGun.hh"
#include "G4RotationMatrix.hh"

// G4 headers including CLHEP headers
// for generating random numbers
#include "Randomize.hh"

// NPTool headers
#include "EventGeneratorTransfert.hh"
#include "RootOutput.h"

using namespace CLHEP;


//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
EventGeneratorTransfert::EventGeneratorTransfert()
{
   //------------- Default Constructor -------------
   m_InitConditions	= new TInitialConditions()	;
   m_Reaction = new Reaction()					;

   m_BeamEnergy       = 0;
   m_BeamEnergySpread = 0;
   m_BeamFWHMX        = 0;
   m_BeamFWHMY        = 0;
   m_BeamEmmitanceTheta      = 0;
   m_BeamEmmitancePhi=0 ;
   m_ShootLight       = 0;
   m_ShootHeavy       = 0;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
EventGeneratorTransfert::~EventGeneratorTransfert()
{
   //------------- Default Destructor ------------
   delete m_InitConditions;
   delete m_Reaction;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
EventGeneratorTransfert::EventGeneratorTransfert(string name1             ,        // Beam nuclei
		             string name2             ,        // Target nuclei
		             string name3             ,        // Product of reaction
		             string name4             ,        // Product of reaction
		             double BeamEnergy        ,        // Beam Energy
		             double ExcitationEnergy  ,        // Excitation of Heavy Nuclei
		             double BeamEnergySpread  ,
		             double BeamFWHMX         ,
		             double BeamFWHMY         ,
		             double BeamEmmitanceTheta       ,
		             double BeamEmmitancePhi  ,
		             bool   ShootLight        ,
		             bool   ShootHeavy        ,
		             string Path)                     // Path of the differentiel Cross Section
{
 SetEverything(		name1             ,        // Beam nuclei
		              name2             ,        // Target nuclei
		              name3             ,        // Product of reaction
		              name4             ,        // Product of reaction
		              BeamEnergy        ,        // Beam Energy
		              ExcitationEnergy  ,        // Excitation of Heavy Nuclei
		              BeamEnergySpread  ,
		              BeamFWHMX         ,
		              BeamFWHMY         ,
		              BeamEmmitanceTheta       ,
		              BeamEmmitancePhi  ,
		               ShootLight        ,
		                ShootHeavy        ,
		             Path);        

}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
void EventGeneratorTransfert::InitializeRootOutput()
{
   RootOutput *pAnalysis = RootOutput::getInstance();
   TTree *pTree = pAnalysis->GetTree();
   pTree->Branch("InitialConditions", "TInitialConditions", &m_InitConditions);
}


//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
void EventGeneratorTransfert::Print() const
{

}


//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
//    Inherit from VEventGenerator

void EventGeneratorTransfert::ReadConfiguration(string Path)
{
////////General Reading needs////////
   string LineBuffer;
   string DataBuffer;

////////Reaction Setting needs///////
   string Beam, Target, Heavy, Light, CrossSectionPath ;
   G4double BeamEnergy = 0 , ExcitationEnergy = 0 , BeamEnergySpread = 0 , BeamFWHMX = 0 , BeamFWHMY = 0 , BeamEmmitanceTheta = 0 , BeamEmmitancePhi=0;
   bool  ShootLight     = false ;
   bool  ShootHeavy      = false ;
   
   bool ReadingStatus = false ;
   bool check_Beam = false ;
   bool check_Target = false ;
   bool check_Light = false ;
   bool check_Heavy = false ;
   bool check_ExcitationEnergy = false ;
   bool check_BeamEnergy = false ;
   bool check_BeamEnergySpread = false ;
   bool check_FWHMX = false ;
   bool check_FWHMY = false ;
   bool check_EmmitanceTheta = false ;
   bool check_EmmitancePhi = false ;
   bool check_CrossSectionPath = false ;
   bool check_ShootLight = false ;
   bool check_ShootHeavy = false ;
   
//////////////////////////////////////////////////////////////////////////////////////////
   ifstream ReactionFile;
   ReactionFile.open(Path.c_str());

   if (ReactionFile.is_open()) {} else {
      return;
   }

   while (!ReactionFile.eof()) {
      //Pick-up next line
      getline(ReactionFile, LineBuffer);

      

      if (LineBuffer.compare(0, 9, "Transfert") == 0) { ReadingStatus = true ;}


while(ReadingStatus){
 			
 			 ReactionFile >> DataBuffer;
 			 
 			 //Search for comment Symbol %
	      	 if (LineBuffer.compare(0, 1, "%") == 0) {/* Do Nothing */;}
 			 
	         else if (DataBuffer.compare(0, 5, "Beam=") == 0) {
	         	check_Beam = true ;
	            ReactionFile >> DataBuffer;
	            Beam = DataBuffer;
	            G4cout << "Beam " << Beam << G4endl;
	         }
	
	         else if (DataBuffer.compare(0, 7, "Target=") == 0) {
	            check_Target = true ;
	            ReactionFile >> DataBuffer;
	            Target = DataBuffer;
	            G4cout << "Target " << Target << G4endl;
	         }

	         else if (DataBuffer.compare(0, 6, "Light=") == 0) {
	         	check_Light = true ;
	            ReactionFile >> DataBuffer;
	            Light = DataBuffer;
	            G4cout << "Light " << Light << G4endl;
	         }

	        else if  (DataBuffer.compare(0, 6, "Heavy=") == 0) {
	            check_Heavy = true ;
	            ReactionFile >> DataBuffer;
	            Heavy = DataBuffer;
	            G4cout << "Heavy " << Heavy << G4endl;
	         }

	        else if  (DataBuffer.compare(0, 17, "ExcitationEnergy=") == 0) {
	        	check_ExcitationEnergy = true ;
	            ReactionFile >> DataBuffer;
	            ExcitationEnergy = atof(DataBuffer.c_str()) * MeV;
	            G4cout << "Excitation Energy " << ExcitationEnergy / MeV << " MeV" << G4endl;
	         }

	        else if  (DataBuffer.compare(0, 11, "BeamEnergy=") == 0) {
	        	check_BeamEnergy = true ;
	            ReactionFile >> DataBuffer;
	            BeamEnergy = atof(DataBuffer.c_str()) * MeV;
	            G4cout << "Beam Energy " << BeamEnergy / MeV << " MeV" << G4endl;
	         }

	        else if  (DataBuffer.compare(0, 17, "BeamEnergySpread=") == 0) {
	        	check_BeamEnergySpread = true ;
	            ReactionFile >> DataBuffer;
	            BeamEnergySpread = atof(DataBuffer.c_str()) * MeV;
	            G4cout << "Beam Energy Spread " << BeamEnergySpread / MeV << " MeV" << G4endl;
	         }

	        else if  (DataBuffer.compare(0, 11, "BeamFWHMX=") == 0) {
	        	check_FWHMX = true ;
	            ReactionFile >> DataBuffer;
	            BeamFWHMX = atof(DataBuffer.c_str()) * mm;
	            G4cout << "Beam FWHM X " << BeamFWHMX << " mm" << G4endl;
	         }

	        else if  (DataBuffer.compare(0, 11, "BeamFWHMY=") == 0) {
	        	check_FWHMY = true ;
	            ReactionFile >> DataBuffer;
	            BeamFWHMY = atof(DataBuffer.c_str()) * mm;
	            G4cout << "Beam FWHM Y " << BeamFWHMX << " mm" << G4endl;
	         }

	        else if  (DataBuffer.compare(0, 19, "BeamEmmitanceTheta=") == 0) {
	        	check_EmmitanceTheta = true ;
	            ReactionFile >> DataBuffer;
	            BeamEmmitanceTheta = atof(DataBuffer.c_str()) * rad;
	            G4cout << "Beam Emmitance Theta " << BeamEmmitanceTheta / deg << " deg" << G4endl;
	         }
	         
	        else if  (DataBuffer.compare(0, 17, "BeamEmmitancePhi=") == 0) {
	        	check_EmmitancePhi = true ;
	            ReactionFile >> DataBuffer;
	            BeamEmmitancePhi = atof(DataBuffer.c_str()) * rad;
	            G4cout << "Beam Emmitance Phi " << BeamEmmitancePhi / deg << " deg" << G4endl;
	         }

	        else if  (DataBuffer.compare(0, 17, "CrossSectionPath=") == 0) {
	        	check_CrossSectionPath = true ;
	            ReactionFile >> CrossSectionPath;
	            G4cout << "Cross Section File: " << CrossSectionPath << G4endl ;
	         }

	        else if  (DataBuffer.compare(0, 11, "ShootLight=") == 0) {
	        	check_ShootLight = true ;
	            ReactionFile >> DataBuffer;
	            if (atof(DataBuffer.c_str()) == 1) ShootLight = true ;
	            if (ShootLight)    G4cout << "Shoot Light particle      : yes" << G4endl;
	            else           G4cout << "Shoot Light particle      : no"  << G4endl;
	         }

	        else if  (DataBuffer.compare(0, 11, "ShootHeavy=") == 0) {
	        	check_ShootHeavy = true ;
	            ReactionFile >> DataBuffer;
	            if (atof(DataBuffer.c_str()) == 1) ShootHeavy = true ;
	            if (ShootHeavy)    G4cout << "Shoot Heavy particle      : yes" << G4endl;
	            else           G4cout << "Shoot Heavy particle      : no"  << G4endl;
	         }

			  
         	///////////////////////////////////////////////////
			//	If no Transfert Token and no comment, toggle out
	         else 
	         	{ReadingStatus = false; G4cout << "WARNING : Wrong Token Sequence: Getting out " << G4endl ;}
	         	
	         ///////////////////////////////////////////////////
			//	If all Token found toggle out
	         if(   	check_Beam && check_Target && check_Light && check_Heavy && check_ExcitationEnergy 
	         	&&  check_BeamEnergy && check_BeamEnergySpread && check_FWHMX && check_FWHMY && check_EmmitanceTheta 
	         	&&  check_EmmitancePhi && check_CrossSectionPath && check_ShootLight && check_ShootHeavy)
	         	ReadingStatus = false ;	

		}
	        

	}
   
   SetEverything(Beam            ,
         Target            ,
         Light          ,
         Heavy          ,
         BeamEnergy        ,
         ExcitationEnergy  ,
         BeamEnergySpread  ,
         BeamFWHMX         ,
         BeamFWHMY         ,
         BeamEmmitanceTheta       ,
         BeamEmmitancePhi	,
         ShootLight        ,
         ShootHeavy        ,
         CrossSectionPath);
         
   		ReactionFile.close();
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
void EventGeneratorTransfert::GenerateEvent(G4Event* anEvent , G4ParticleGun* particleGun)
{	
   // Clear contents of Precedent event (now stored in ROOTOutput)
   m_InitConditions->Clear();

   //////////////////////////////////////////////////
   //////Define the kind of particle to shoot////////
   //////////////////////////////////////////////////
   G4int LightZ = m_Reaction->GetNucleus3()->GetZ() ;
   G4int LightA = m_Reaction->GetNucleus3()->GetA() ;

   G4ParticleDefinition* LightName
   = G4ParticleTable::GetParticleTable()->GetIon(LightZ, LightA, 0.);

   G4int HeavyZ = m_Reaction->GetNucleus4()->GetZ() ;
   G4int HeavyA = m_Reaction->GetNucleus4()->GetA() ;

   G4ParticleDefinition* HeavyName
   = G4ParticleTable::GetParticleTable()->GetIon(HeavyZ, HeavyA, m_Reaction->GetExcitation()*MeV);

   // Vertex position and beam angle inte world frame
   G4double x0 = 1000 * cm  ;
   G4double y0 = 1000 * cm  ;
   G4double Beam_thetaX = 0  ;
   G4double Beam_phiY   = 0  ;
   
   //shoot inside the target with correlated angle
   if (m_TargetRadius != 0) {
      while (sqrt(x0*x0 + y0*y0) > m_TargetRadius) 
      	{
      		RandomGaussian2D(0,0,m_BeamFWHMX / 2.35,m_BeamEmmitanceTheta,x0,Beam_thetaX);
      		RandomGaussian2D(0,0,m_BeamFWHMY / 2.35,m_BeamEmmitancePhi  ,y0,Beam_phiY  );
      	}
   }

   else 
   	{
     	RandomGaussian2D(0,0,0,m_BeamEmmitanceTheta,x0,Beam_thetaX);
     	RandomGaussian2D(0,0,0,m_BeamEmmitancePhi  ,y0,Beam_phiY  );
   }

	// Calculate Angle in spherical coordinate, passing by the direction vector dir	
	G4double Xdir =  cos( pi/2. - Beam_thetaX ) 							;
	G4double Ydir =  cos( pi/2. - Beam_phiY   )								;
	G4double Zdir =  sin( pi/2. - Beam_thetaX ) + sin(  pi/2. - Beam_phiY) 	;
	
	G4double Beam_theta = acos ( Zdir / sqrt( Xdir*Xdir + Ydir*Ydir + Zdir*Zdir ) );
	G4double Beam_phi   = atan2( Ydir , Xdir );   

   // write angles to ROOT file
   m_InitConditions->SetICIncidentAngleTheta(Beam_theta / deg);
   m_InitConditions->SetICIncidentAnglePhi(Beam_phi / deg);

   //////////////////////////////////////////////////////////
   ///// Build rotation matrix to go from the incident //////
   ///// beam frame to the "world" frame               //////
   //////////////////////////////////////////////////////////
   G4ThreeVector col1(cos(Beam_theta) * cos(Beam_phi),
                      cos(Beam_theta) * sin(Beam_phi),
                      -sin(Beam_theta));
   G4ThreeVector col2(-sin(Beam_phi),
                      cos(Beam_phi),
                      0);
   G4ThreeVector col3(sin(Beam_theta) * cos(Beam_phi),
                      sin(Beam_theta) * sin(Beam_phi),
                      cos(Beam_theta));
   G4RotationMatrix BeamToWorld(col1, col2, col3);

   /////////////////////////////////////////////////////////////////
   ///// Angles for emitted particles following Cross Section //////
   ///// Angles are in the beam frame                         //////
   /////////////////////////////////////////////////////////////////
   // Beam incident energy
   G4double NominalBeamEnergy = m_BeamEnergy;
   G4double IncidentBeamEnergy = RandGauss::shoot(NominalBeamEnergy, m_BeamEnergySpread / 2.35);
   m_Reaction->SetBeamEnergy(IncidentBeamEnergy);
   m_InitConditions->SetICIncidentEnergy(IncidentBeamEnergy / MeV);
   // Angles
   RandGeneral CrossSectionShoot(m_Reaction->GetCrossSection(), m_Reaction->GetCrossSectionSize());
   G4double ThetaCM = CrossSectionShoot.shoot() * (180*deg);
   G4double phi     = RandFlat::shoot() * 2*pi;
   // write angles to ROOT file
   m_InitConditions->SetICEmittedAngleThetaCM(ThetaCM / deg);
   m_InitConditions->SetICEmittedAnglePhiIncidentFrame(phi / deg);

   //////////////////////////////////////////////////
   /////  Momentum and angles from  kinematics  /////
   /////  Angles are in the beam frame          /////
   //////////////////////////////////////////////////
   // Variable where to store results
   G4double ThetaLight, EnergyLight, ThetaHeavy, EnergyHeavy;
   // Set the Theta angle of reaction
   m_Reaction->SetThetaCM(ThetaCM);
   // Compute Kinematic using previously defined ThetaCM
   m_Reaction->KineRelativistic(ThetaLight, EnergyLight, ThetaHeavy, EnergyHeavy);
   // Momentum in beam frame for light particle
   G4ThreeVector momentum_kineLight_beam(sin(ThetaLight) * cos(phi),
                                         sin(ThetaLight) * sin(phi),
                                         cos(ThetaLight));
   // Momentum in beam frame for heavy particle
   G4ThreeVector momentum_kineHeavy_beam(sin(ThetaHeavy) * cos(phi),
                                         sin(ThetaHeavy) * sin(phi),
                                         cos(ThetaHeavy));

   // write angles/energy to ROOT file
   m_InitConditions->SetICEmittedAngleThetaLabIncidentFrame(ThetaLight / deg);
   m_InitConditions->SetICEmittedEnergy(EnergyLight);

   //must shoot inside the target.
   G4double z0 = (-m_TargetThickness / 2 + RandFlat::shoot() * m_TargetThickness);

   // Move to the target
   x0 += m_TargetX ;
   y0 += m_TargetY ;
   z0 += m_TargetZ ;

   // write vertex position to ROOT file
   m_InitConditions->SetICPositionX(x0);
   m_InitConditions->SetICPositionY(y0);
   m_InitConditions->SetICPositionZ(z0);

   //////////////////////////////////////////////////
   ///////// Set up everything for shooting /////////
   //////////////////////////////////////////////////
   if (m_ShootLight) {   // Case of light particle
      // Particle type
      particleGun->SetParticleDefinition(LightName);
      // Particle energy
      particleGun->SetParticleEnergy(EnergyLight);
      // Particle vertex position
      particleGun->SetParticlePosition(G4ThreeVector(x0, y0, z0));
      // Particle direction
      // Kinematical angles in the beam frame are transformed 
      // to the "world" frame
      G4ThreeVector momentum_kine_world = BeamToWorld * momentum_kineLight_beam;
      // get theta and phi in the world frame
      G4double theta_world = momentum_kine_world.theta();
      G4double phi_world   = momentum_kine_world.phi();
      if (phi_world < 1e-6) phi_world += 2*pi;
      // write angles in ROOT file
      m_InitConditions->SetICEmittedAngleThetaLabWorldFrame(theta_world / deg);
      m_InitConditions->SetICEmittedAnglePhiWorldFrame(phi_world / deg);
      // tests
//      G4cout << "XXXXXXXXXXXXXXXXXXXXXXXXXXXX" << G4endl;
//      G4cout << "cinematique dans ref world : " << G4endl;
//      G4cout << "\t" << momentum_kine_world << G4endl;
      //Set the gun to shoot
      particleGun->SetParticleMomentumDirection(momentum_kine_world);
      //Shoot the light particle
      particleGun->GeneratePrimaryVertex(anEvent);
   }
   if (m_ShootHeavy) { // Case of recoil particle
      //
      // The case of recoil particle has not been tested with the new version of the event generator
      //
      // Particle type
      particleGun->SetParticleDefinition(HeavyName);
      // Particle energy
      particleGun->SetParticleEnergy(EnergyHeavy);
      // Particle vertex position
      particleGun->SetParticlePosition(G4ThreeVector(x0, y0, z0));
      // Particle direction
      // Kinematical angles in the beam frame are transformed 
      // to the "world" frame
      G4ThreeVector momentum_kine_world = BeamToWorld * momentum_kineHeavy_beam;
      // get theta and phi in the world frame
      G4double theta_world = momentum_kine_world.theta();
      G4double phi_world   = momentum_kine_world.phi();
      if (phi_world < 1e-6) phi_world += 2*pi;
      // write angles in ROOT file
      m_InitConditions->SetICEmittedAngleThetaLabWorldFrame(theta_world / deg);
      m_InitConditions->SetICEmittedAnglePhiWorldFrame(phi_world / deg);
      // tests
//      G4cout << "XXXXXXXXXXXXXXXXXXXXXXXXXXXX" << G4endl;
//      G4cout << "cinematique dans ref world : " << G4endl;
//      G4cout << "\t" << momentum_kine_world << G4endl;
      //Set the gun to shoot
      particleGun->SetParticleMomentumDirection(momentum_kine_world);
      //Shoot the light particle
      particleGun->GeneratePrimaryVertex(anEvent);
   }
}


//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
void EventGeneratorTransfert::SetEverything(string name1             ,        // Beam nuclei
		             string name2             ,        // Target nuclei
		             string name3             ,        // Product of reaction
		             string name4             ,        // Product of reaction
		             double BeamEnergy        ,        // Beam Energy
		             double ExcitationEnergy  ,        // Excitation of Heavy Nuclei
		             double BeamEnergySpread  ,
		             double BeamFWHMX         ,
		             double BeamFWHMY         ,
		             double BeamEmmitanceTheta       ,
		             double BeamEmmitancePhi  ,
		             bool   ShootLight        ,
		             bool   ShootHeavy        ,
		             string Path) 
{
   m_Reaction = new Reaction(	name1					, 
				name2					,
	 			name3					,
	 			name4					,
				BeamEnergy 				,
				ExcitationEnergy				,
				Path 					);	
		
   m_BeamEnergy       =  BeamEnergy        ;
   m_BeamEnergySpread =  BeamEnergySpread  ;
   m_BeamFWHMX        =  BeamFWHMX         ;
   m_BeamFWHMY        =  BeamFWHMY         ;
   m_BeamEmmitanceTheta      =  BeamEmmitanceTheta       ;
   m_BeamEmmitancePhi      =  BeamEmmitancePhi      ;
   m_ShootLight       =  ShootLight        ;
   m_ShootHeavy       =  ShootHeavy        ;
}

